@@ -106,7 +106,11 @@ bmem.sobel.ind<-function(sem.object, ind){
 	first.deriv<-eval(ind.deriv, par.list)
 	first.deriv<-attributes(first.deriv)$gradient
 	
-	var.par<-sem.object$cov[par,par]
+	if (is.null(sem.object$cov)) {
+		var.par<-sem.object$vcov[par,par]
+		}else{
+			var.par<-sem.object$cov[par,par]
+		}
 	var.ind<-first.deriv%*%var.par%*%t(first.deriv)
 	s.e.ind<-sqrt(var.ind)
 	
@@ -1052,3 +1056,111 @@ plot.bmem<-function(x, par, ...){
 		lines(density(x$boot.est$boot.fit[,par]))
 	}	
 }
+
+bmem.cov <- function(ram,obs.variables,moment=FALSE, debug=FALSE){
+	if (moment){
+		fixed.x<-'(i)'
+	}else{
+		fixed.x=NULL
+	}
+    parse.path <- function(path) {                                           
+        path.1 <- gsub('-', '', gsub(' ','', path))
+        direction <- if (regexpr('<>', path.1) > 0) 2 
+            else if (regexpr('<', path.1) > 0) -1
+            else if (regexpr('>', path.1) > 0) 1
+            else stop(paste('ill-formed path:', path))
+        path.1 <- strsplit(path.1, '[<>]')[[1]]
+        list(first=path.1[1], second=path.1[length(path.1)], direction=direction)
+        }
+    if ((!is.matrix(ram)) | ncol(ram) != 3) stop ('ram argument must be a 3-column matrix')
+    startvalues <- as.numeric(ram[,3])
+    par.names <- ram[,2]
+    n.paths <- length(par.names)
+    heads <- from <- to <- rep(0, n.paths)
+    for (p in 1:n.paths){
+        path <- parse.path(ram[p,1])
+        heads[p] <- abs(path$direction)
+        to[p] <- path$second
+        from[p] <- path$first
+        if (path$direction == -1) {
+            to[p] <- path$first
+            from[p] <- path$second
+            }
+        }
+    ram <- matrix(0, p, 5)
+    all.vars <- unique(c(to, from))
+    latent.vars <- setdiff(all.vars, obs.variables)
+    not.used <- setdiff(obs.variables, all.vars)
+    
+    vars <- c(obs.variables, latent.vars)
+    pars <- na.omit(unique(par.names))
+    ram[,1] <- heads
+    ram[,2] <- apply(outer(vars, to, '=='), 2, which)
+    ram[,3] <- apply(outer(vars, from, '=='), 2, which)   
+    par.nos <- apply(outer(pars, par.names, '=='), 2, which)
+    if (length(par.nos) > 0)
+        ram[,4] <- unlist(lapply(par.nos, function(x) if (length(x) == 0) 0 else x))
+    ram[,5]<- startvalues
+    colnames(ram) <- c('heads', 'to', 'from', 'parameter', 'start')
+    if (!is.null(fixed.x)) fixed.x <- apply(outer(vars, fixed.x, '=='), 2, which)
+    n <- length(obs.variables)
+    m <- length(all.vars)
+    t <- length(pars)
+    if (debug) {
+        cat('\n observed variables:\n') 
+        print(paste(paste(1:n,':', sep=''), obs.variables, sep=''))
+        cat('\n')
+        if (m > n){ 
+            cat('\n latent variables:\n')
+            print(paste(paste((n+1):m,':', sep=''), latent.vars, sep=''))
+            cat('\n')
+            }
+        cat('\n parameters:\n') 
+        print(paste(paste(1:t,':', sep=''), pars, sep=''))
+        cat('\n\n RAM:\n')
+        print(ram)
+        }
+    n<-length(obs.variables)
+    observed <- 1:n
+    n.fix <- length(fixed.x)
+    
+    m <- max(ram[,c(2,3)])
+    missing.variances <- setdiff(1:m, ram[,2][ram[,2] == ram[,3]])
+    if (length(missing.variances) > 0) warning(paste( "\nThe model is almost surely misspecified; check also for missing covariances.\n"))
+    t <- max(ram[,4])
+    df <- n*(n + 1)/2 - t - n.fix*(n.fix + 1)/2
+    if (df < 0) stop(paste("The model has negative degrees of freedom =", df))
+    J <- matrix(0, n, m)
+    correct <- matrix(2, m, m)
+    diag(correct) <- 1
+    J[cbind(1:n, observed)]<-1
+    par.posn <-  sapply(1:t, function(i) which(ram[,4] == i)[1])
+    colnames(ram)<-c("heads", "to", "from", "parameter", "start value")
+    rownames(ram)<-rep("",nrow(ram))
+    #if (length(param.names) > 0) rownames(ram)[par.posn]<-param.names
+    fixed <- ram[,4] == 0
+    sel.free <- ram[,4]
+    sel.free[fixed] <- 1
+    one.head <- ram[,1] == 1
+    one.free <- which( (!fixed) & one.head )
+    two.free <- which( (!fixed) & (!one.head) )
+    arrows.1 <- ram[one.head, c(2,3), drop=FALSE]
+    arrows.2 <- ram[!one.head, c(2,3), drop=FALSE]
+    arrows.2t <- ram[!one.head, c(3,2), drop=FALSE]
+    arrows.1.free <- ram[one.free,c(2,3), drop=FALSE]
+    arrows.2.free <- ram[two.free,c(2,3), drop=FALSE]
+    sel.free.1 <- sel.free[one.free]
+    sel.free.2 <- sel.free[two.free]
+    unique.free.1 <- unique(sel.free.1)
+    unique.free.2 <- unique(sel.free.2)
+    
+    A <- P <- matrix(0, m, m)
+    val <- ifelse (fixed, ram[,5], 0)
+    A[arrows.1] <- val[one.head]
+    P[arrows.2t] <- P[arrows.2] <- val[!one.head]
+    I.Ainv <- solve(diag(m) - A)
+    C <- J %*% I.Ainv %*% P %*% t(I.Ainv) %*% t(J)
+    rownames(C)<-colnames(C)<-obs.variables
+    C
+    }
+    
\ No newline at end of file
